#include <iostream>

char *tablero =
    "000900600009056002000000008070041000010600090005000007700200000540000006601408700";

class Casilla {
    private:
    int valor;

    public:
    Casilla() : valor(0) {}
    int Valor() const { return valor; }
    void Asignar(int v) { valor = v; }
    void Mostrar() const { std::cout << valor << " "; }
};

class Tablero;

class Fila {
    private:
    Casilla *cas[9];

    public:
    Fila() { for(int i=0; i < 9; i++) cas[i] = NULL; }
    void Iniciar(Tablero &t, int c);
    bool Presente(int n) const;
};

class Columna {
    private:
    Casilla *cas[9];

    public:
    Columna() { for(int i=0; i < 9; i++) cas[i] = NULL; }
    void Iniciar(Tablero &t, int f);
    bool Presente(int n);
};

class Caja {
    private:
    Casilla *cas[9];

    public:
    Caja() { for(int i=0; i < 9; i++) cas[i] = NULL; }
    void Iniciar(Tablero &t, int c);
    bool Presente(int n);
};

class Tablero {
    private:
    Casilla casilla[9][9];
    Fila fila[9];
    Columna columna[9];
    Caja caja[9];
    int cuenta;

    public:
    Tablero(char* t);
    Casilla &GetCasilla(int f, int c) { return casilla[f][c]; }
    const int Cuenta() { return cuenta; }
    void Mostrar() const;
    bool Soluciones();
    bool Valido(int f, int c, int v);
};

void Fila::Iniciar(Tablero &t, int f) {
    for(int i = 0; i < 9; i++) cas[i] = &t.GetCasilla(f, i);
}

bool Fila::Presente(int n) const {
    for(int i=0; i<9; i++) if(cas[i]->Valor() == n) return true;
    return false;
}

void Columna::Iniciar(Tablero &t, int c) {
    for(int i = 0; i < 9; i++) cas[i] = &t.GetCasilla(i, c);
}

bool Columna::Presente(int n) {
    for(int i=0; i<9; i++) if(cas[i]->Valor() == n) return true;
    return false;
}

void Caja::Iniciar(Tablero &t, int c) {
    for(int i = 0; i < 3; i++)
        for(int j = 0; j < 3; j++)
            cas[i*3+j] = &t.GetCasilla((c/3)*3+i, (c%3)*3+j);
}

bool Caja::Presente(int n) {
    for(int i=0; i<9; i++) if(cas[i]->Valor() == n) return true;
    return false;
}

Tablero::Tablero(char* t) : cuenta(0) {
    int f, c;

    for(f=0; f < 9; f++)
        for(c=0; c < 9; c++) {
            casilla[f][c].Asignar(t[f*9+c]-'0');
        }
    for(f=0; f < 9; f++) fila[f].Iniciar(*this, f);
    for(c=0; c < 9; c++) columna[c].Iniciar(*this, c);
    for(c=0; c < 9; c++) caja[c].Iniciar(*this, c);
}

void Tablero::Mostrar() const {
    for(int f=0; f<9; f++) {
        if(f%3==0 && f != 0) std::cout << "------+------+------" << std::endl;
        for(int c=0; c<9; c++) {
            if(c%3==0 && c != 0) std::cout << "|";
            casilla[f][c].Mostrar();
        }
        std::cout << std::endl;
    }
}

bool Tablero::Valido(int f, int c, int v) {
    return !fila[f].Presente(v) && !columna[c].Presente(v) && !caja[(f/3)*3+(c/3)].Presente(v);
}

bool Tablero::Soluciones() {
    int f, c, n, s;
    for(f=0; f<9; f++) {
        for(c=0; c<9; c++) {
            if(casilla[f][c].Valor() == 0) {
                for(n=1; n<=9; n++) {
                    if(Valido(f, c, n)) {
                        casilla[f][c].Asignar(n);
                        if(Soluciones()) return true;
                        else casilla[f][c].Asignar(0);
                    }
                }
                return false;
            }
        }
    }
    cuenta++;
    return false;
}

int main()
{
    Tablero T(tablero);
    T.Mostrar();

    T.Soluciones();
    std::cout << "Soluciones: " << T.Cuenta() << std::endl;
    return 0;
}
